#!/usr/bin/env perl

# main.command
# LookUpNDSHomeDirAction

#  Created by Tim McLaughlin on 2005-06-08.
#  Copyright 2005 __MyCompanyName__. All rights reserved.

#!/usr/bin/perl

$main::DEBUG = 0;

#
## Setup ##

use IO::File;
use IO::Socket;

my $swd;
my $LOG_PATH = "/tmp/ndsthing.txt";

require "ldsHelper.pl";
#DMH: require "$swd/ldsHelper.pl";

# Bytes to send for the AFP-Over-TCP handshake
my @send = map { chr(hex($_)); } qw(00 03 00 01 00 00 00 00 00 00 00 02 00 00 00 00 0f 00);
my $afpHandshake = join('', @send);
undef @send;

## End Setup ##
#


#
## Subroutines ##

sub BEGIN {
    # Get the current scriptWorkingDirectory
    my @parts = split( '/', $0 );
    pop( @parts );
    $swd = join( '/', @parts );
}


sub END {
    # add the 'end' message to the log, then close it
    logMesg( '==== End Run On ' . localtime() . ' ====' . "\n" );
    undef $main::logfh;
}


# This is how to log a message
sub logMesg ($) {
    my $message = shift;

    if ($main::logfh == undef) {
        # Open the log file for appending
        $main::logfh = IO::File->new(">> ${LOG_PATH}")
            or warn "Couldn't open ${LOG_PATH}: $!\n";

        # This disables buffering on our log filehandle
        my $old_fh = select($main::logfh);
        $| = 1;
        select($old_fh);
    }

    # Print the message to the log file
    print $main::logfh ($message . "\n");
}


# This sends an AFP "handshake" to the intended Netware server, to
# determine which server is actually answering for the service.  The
# only reason this is important is that Mac OS X seems to want to use
# "FACSRV2.USER2" rather than just "USER2" for the volume name.  This
# subroutine returns the FACSRV# or STUSRV# part of the volume name.
# The handshake response from the server includes the IP address of
# the server that is actually responding.  This is used to look up the
# information from ou=Services, o=wwu to find the human-readable name.
sub getServerName($$) {
    my $afpHandshake = shift;
    my $peerAddr = shift;

    my $socket = IO::Socket::INET->new(PeerAddr => $peerAddr,
                                      PeerPort => 548,
                                      Proto    => 'tcp',
                                      Type     => SOCK_STREAM)
        or die "Couldn't connect to remote host: $@\n";

    print $socket $afpHandshake;
    my $found = undef;
    my $answer;
    while ( !$found && !eof($socket) ) {
        $answer = <$socket>;
        logMesg("          answer is $answer") if ($main::DEBUG);
        $answer =~ m/(2-Way Randnum exchange|Cleartxt passwrd)(140\.160\.246\.\d{2})/i;
        $found = $2 if ($2);
    }

    close($socket);

    # Grok the AUTH type
    my $authType = 'NO%20USER%20AUTHENT';
    if ($1 eq '2-Way Randnum exchange') {
        $authType = '2-WAY%20RANDNUM';
    }
    elsif ($1 eq 'Cleartxt passwrd') {
        $authType = 'CLEARTXT%20PASSWRD';
    }

    # Now we know the IP, so look up the name ((FAC|STU)SRV\d)
    my $srvip = $found;
    $mesg = ldap::search( "SU=WWUSLP, ou=SLP-MGMT, ou=Services, o=wwu",
                          "sub",
                          "(slpurl=afp://${srvip}/*)",
                          [('slpurl')]
                        );

    # If we got results, return them.  Otherwise return undef.
    if ( $mesg->{'slpurl'} ) {
        $mesg->{'slpurl'} =~ m/NAME=((FAC|STU)SRV\d)/i;
        return $1;
    }

    return undef;
}


# We are provided with the username, but we need to know which context
# the user is in, so that we can look up various information stemming
# from their homeDirectory entry.  This subroutine returns the full
# user DN.  e.g. cn=vikingv,ou=Students,o=wwu
sub findUserDn ($)
{
    my $username = shift;

    # Find the user's DN
    $mesg = ldap::search( "o=wwu",
                          "sub", 
                          "(cn=$username)",
                          [('ndsHomeDirectory')]
                        );

    # If we got results, return them.  Otherwise return undef.
    if ( $mesg ) {
        return $mesg;
    }

    return undef;
}


# The following three subroutines, getHostServer, getResource and
# getLoadScript, are used by getUrlForUser to follow a chain of information
# stemming from the ndsHomeDirectory attribute of the user's record.
# All three are just pretty basic DN lookups.
sub getHostServer ($)
{
    my $homeDir = shift;

    # Split up the incoming
    $homeDir =~ m/([^,]+),(.*)/;
    ($p1, $p2) = ($1, $2);

    # 
    $mesg = ldap::search( $p2,
                          "sub",
                          "($p1)",
                          [('hostServer')]
                        );

    # If we got results, return them.  Otherwise return undef.
    if ( $mesg ) {
        return $mesg;
    }

    return undef;
}

# See description above getHostServer
sub getResource ($)
{
    my $hostServer = shift;

    # Split up the incoming
    $hostServer =~ m/([^,]+),(.*)/;
    ($p1, $p2) = ($1, $2);

    # 
    $mesg = ldap::search( $p2,
                          "sub",
                          "($p1)",
                          [('Resource')]
                        );

    # If we got results, return them.  Otherwise return undef.
    if ( $mesg ) {
        return $mesg;
    }

    return undef;
}

# See description above getHostServer
sub getLoadScript ($)
{
    my $resource = shift;

    # Split up the incoming
    $resource =~ m/([^,]+),(.*)/;
    ($p1, $p2) = ($1, $2);

    # 
    $mesg = ldap::search( $p2,
                          "sub",
                          "($p1)",
                          [('nCSCRMLoadScript')]
                        );

    # If we got results, return them.  Otherwise return undef.
    if ( $mesg ) {
        return $mesg;
    }

    return undef;
}


# This is where most of the work is done.  Starts with just a username, and
# ends up with an AFP URL for the user's Novell home directory.
sub getUrlForUser ($)
{
    my $userName = shift;

    # first off, get the ndsHomeDirectory attribute, and strip the
    my $userRecord = findUserDn($userName);
    if ($userRecord) {
        # We got the userRecord
        $userDn = $userRecord->{'dn'};
    
        # Print some things out for debugging
        logMesg("              dn is $userDn");
    
        # Figure out the homeDir and homeVolume values
        $userRecord->{'ndsHomeDirectory'}
            =~ m/(cn=WUF-(FAC|STU)-(.*(\d+)),(ou=Servers,ou=Students|ou=its),o=wwu)/i;
        $homeDir    = $1;   # Should look like cn=WUF-FAC-USER2,ou=its,o=wwu
        $homeVolume = $3;   # Should look like USER2
    
        # Print some things out for debugging
        logMesg("ndsHomeDirectory is $homeDir");
        logMesg("      homeVolume is $homeVolume");
    }
    else {
        logMesg("No userRecord found for $userName");
        die("No userRecord found for $userName");
    }
    
    
    # Get the hostServer for the homeDir
    $hostServerRecord = getHostServer($homeDir);
    if ($hostServerRecord) {
        # We got the hostServerRecord
        # Get the hostServer attribute
        # Should look like cn=WUF-USER2,ou=its,o=wwu
        $hostServer = $hostServerRecord->{'hostServer'};

        $hostServer =~ m/cn=([^,]+),/i;
        $fqdn = "\L$1\E.wuf.wwu.edu";

        # Print some things out for debugging
        logMesg("      hostServer is $hostServer");
        logMesg("        hostFQDN is $fqdn");
    }
    else {
        logMesg("No hostServerRecord found");
        die("No hostServerRecord found");
    }
    
    # Get the server that the homeVolume resides on
    # Should look like FACSRV2 or STUSRV2
    my $homeServer = getServerName($afpHandshake, $fqdn);

    # Print some things out for debugging
    logMesg("      homeServer is $homeServer");

    # Create the user string from the userDn
    $ndsUserString = '';
    my @dnParts = split(',', $userDn);
    foreach $part (@dnParts) {
        $ndsUserString .= '.' . (split('=', $part))[1];
    }

    # Create the URL and return it
    # $url = qq|afp://$ndsUserString;AUTH=$authType\@$fqdn/$homeServer.$homeVolume/$userName|;
    $url = qq|afp://$ndsUserString\@$fqdn/$homeServer.$homeVolume/$userName|;
    logMesg("End Result: $url");
    
    return $url;
}

## End Subroutines ##
#


#
## Main ##

# Begin the log entry
logMesg( "\n" . '==== Begin Run On ' . localtime() . ' ====' );

# Get the passed-in userName
#DMH: my @lines = <STDIN>;
#DMH:my $userName = $lines[0];
#DMH:chomp $userName;

#DMH:foreach ( @lines ) {
#DMH:	logMesg( "line: " . $_ );
#DMH:}

$userName = $ARGV[0];

# Wrap the code in an eval so we can catch exceptions
eval {
    # Get the url for this user
    my $url = getUrlForUser($userName);
    print $url."\n";
};
# Check for exceptions
if ($@) {
    logMesg("Couldn't create My Home.url file");
    exit();
}

exit();

## End Main ##
#
